/*---------------------------------------------------------------------------
      This code is part of the book
      LOGIC PROGRAMMING TECHNIQUES: THE PROLOG LANGUAGE
      Ilias Sakellariou, Nikos Bassiliades, Petros Kefalas, Demosthenes Stamatis
      Hellenic Academic On-line Publications, 2015
    ----------------------------------------------------------------------------*/
    

% count/3
% count(E,L,N)
% element E exists N times inside list L

% First version. 
% Unification is used for checking
% if the element exists in the list
count1(_,[],0).
count1(A,[A|L],N) :- 
	count1(A,L,N1), 
	N is N1 + 1.
count1(A,[H|L],N) :- 
	A \= H,
	count1(A,L,N).

% Second version.
% Term equality is used to check 
% if the element exists in the list
count2(_,[],0).
count2(A,[|L],N) :- 
	 == ,
	count2(A,L,N1), 
	N is N1 + 1.
count2(A,[H|L],N) :- 
	A \== H,
	count2(A,L,N).

% Third version. 
% Unification is used for checking
% if the element exists in the list
% but only when both the element and the head of list
% are not variables
count3(_,[],0).
count3(A,[|L],N) :- 
	nonvar(A), nonvar(H),  = ,
	count3(A,L,N1), 
	N is N1 + 1.
count3(A,[H|L],N) :- 
	A \= H,
	count3(A,L,N).
