/*---------------------------------------------------------------------------
      This code is part of the book
      LOGIC PROGRAMMING TECHNIQUES: THE PROLOG LANGUAGE
      Ilias Sakellariou, Nikos Bassiliades, Petros Kefalas, Demosthenes Stamatis
      Hellenic Academic On-line Publications, 2015
    ----------------------------------------------------------------------------*/
    


% my_functor/3
my_functor(Term,Functor,Arity) :-
	Term =.. List,			% Decompose Term
	List = [Functor|Arguments],	% Separate head-functor and Tail-arguments
	length(Arguments,Arity).	% count how many arguments (elements in tail)

% my_arg/3
my_arg(N,Term,Arg) :-
	Term =.. List,			% Decompose Term
	List = [Functor|Arguments],	% Separate head-functor and Tail-arguments
%	nth_member(N,Arg,Arguments).	% Return nth member of the list / check nth-member.pl
	nth1(N,Arguments,Arg).		% or as in SWI-Prolog built-in

% univ/2
univ(Term,List) :-
	nonvar(Term),				% If the Term is not a variable
	functor(Term,Functor,Arity),		% De-Compose the Term into Functor/Arity
	univ_aux(Term,1,Arity,Arguments),	% unify recursively each argument of the term with the corresponding list element
	List = [Functor| Arguments].		% Compose the list with the Argument list and the Functor as head
univ(Term,List) :-
	var(Term),				% If the Term is a variable
	List = [Functor| Arguments],		% Then the list is decomposed to head/functor and tail/arguments
	length(Arguments,Arity),		% the arity is the length of the tail/arguments
	functor(Term,Functor,Arity),		% compose a term from functor and arity
	univ_aux(Term,1,Arity,Arguments).	% unify recursively each argument of the term with the corresponding list element

% univ_aux/3
% univ_aux(Term,N,Arity,Arguments)
% Unifies recursively each argument N of the Term with the corresponding list element
% of the list of Arguments, until N reaches the arity
univ_aux(_,N,Arity,[]) :-			% Terminal condition when N exceeds arity and the are no more arguments in the list
	N > Arity.
univ_aux(Term,N,Arity,[Arg|Rest]) :-		% When there a elements in the argument list 
	N =< Arity,				
	arg(N,Term,Arg),			% unify N-th argument of Term with the N-th list element
	N1 is N + 1,				% move to the next element / argument
	univ_aux(Term,N1,Arity,Rest).		% and iterate (recurse)
